==> WeefPy-1.0.4.tar.gz: WeefPy-1.0.4/README.md <==
# WeefPy - Tiny Python web API framework

Tiny WSGI-Python framework designed mainly for REST API backends

## Installing WeefPy

Simply use pip for install this package
Installation:
```sh
$ pip install WeefPy
```
If you want to use development server install its dependencies
```sh
$ pip install uwsgi argparse python-daemon
```
## Running development server

Change directory to project folder and run devserver.

```sh
$ weef_server --host 127.0.0.1 --port 8080 --app `file`:`callable`
```
Replace `file` with filename with your main function and `callable` with your function. <br>
App should run on [localhost:8080](http://localhost:8080/).

#### Command parameters
- `--config` should contain uwsgi ini configuration file.
- `--host` should contain IP address to be server binded on.
- `--port` should contain port to be server binded on.
- `--app` should contain file and callable object separated by colon. (main.py:main)
- `--pythonpath` should contain your code directory for imports in your project
- `--daemon` starts server in deamon mode.

### Bash syntax:
```sh
$ weef_server [--config CONFIG] [--host IP_ADDRESS] [--port PORT] [--app file:callable] [--pythonpath path] [--daemon]
```

You can create server configuration in uwsgi ini file. <br>
If you want to run python files without devserver add project directory to `PYTHONPATH`.

## Using WeefPy

### Initializing

```py
from WeefPy import WSGI

application = WSGI()

```
#### WSGI parameters:
- default_content_type - this argument specifies the default mime type of generated responses
- status_codes - this argument specifies dictionary that is used to determine HTTP answers for 4xx & 5xx codes in different content types - by default is used text/html type which is mandatory if this argument is specified. For example watch [codes.json](https://gitlab.com/katry/WeefPy/-/blob/master/WeefPy/codes.json).

### Routes

#### Route parameters
- endpoint - object should contain str or regex Pattern to match request path - mandatory
- methods - list of HTTP methods to match request method - higher priority than method, optional - empty list by default
- method - single HTTP method to match request method - optional default value is `GET`

#### Using
Route is used as a decorator callable object you want to use as router method.
Router method takes two mandatory arguments `Request` and `Response` (explained below in a separate section).
Router method can also take variables from router regex match (if used).

#### Examples

```py
from WeefPy import Route


@Route("/endpoint", method="GET")  # simple route
def routed_method_1(request, response):  # simple route method
	return "Response from route 1" # response return without status code & mime type


# re package import needed
@Route(re.compile(r"/endpoint/(\d+)$"), methods=["GET", "POST"]) # regex route, with multiple mthods
def routed_method_2(request, response):
	return "Response from route 2", "200 OK", "text/html" # full response return


@Route("/endpoint3", methods=["GET", "POST"]) #simple route, with multiple methods
def routed_method_2(request, response):
	response.set() # alternatively can be called reponse method set to set response
```

### Request & Response objects

Instances of these object are given to every function that is called with `@Route` decorator
```py
@Route("/endpoint", method="GET")
def endpoint(Request, Response):  # instances of Request and Response objects
	pass
```

#### Request items and methods
- host - host recieved message (ip, domain etc.)
- method - request method (GET, POST etc.)
- path - request path
- query_string - request query_string
- data - request parsed data (json, -www-form-urlencoded, multipart/form-data)
- content_type - request content type
- content_length - request content length
- cookie - `SimpleCookie` object instance with request cookies loaded
<br><br>
- get_header(name) - get HTTP header by name

#### Response  items and methods
- res - `wsgi response` object
- encoding - response encoding; default: `UTF-8`
- headers - response headers
<br><br>
- get_body() - get response body
- get_status() - get response status
- get_content_type() - response content type - by default inherited from `WSGI object`
- set(body, status, content_type) - sets response (`body` (mandatory), `status` - by defaul is set to `200 OK`, `content_type` - if not provided is inherited from `WSGI object content_type`)
- redirect(location, status) - redirects response to (`location`, `status` if not providet is set to `302`)
- set_cookie(name, value, expires=None, maxAge=None, **kwargs) - adds cookie to response with obvious parameres, you can alwo add additional arguments (`kwargs`) such as `Domain`, `Path`, `Secure`, `HttpOnly`
- add_header(name, value) - adds header to response

## Example

Save this file as main.py
```py
from WeefPy import WSGI, Route

application = WSGI(default_content_type="text/html")


@Route("/endpoint", method="GET")
def endpoint(Request, Response):
	Response.add_header("X-header", "blahblah")  # set response header
	Response.set_cookie("cookie_from_server", "384")  # set reponse cookie
	return "Response BODY"
```

Now you should run weef_server
```sh
$ weef_server --host 127.0.0.1 --port 8080 --app main.py:application
```
App should run on [localhost:8080](http://localhost:8080/).<br>
http://localhost:8080/endpoint should now returns "Response BODY"

==> WeefPy-1.1.0.tar.gz: WeefPy-1.1.0/README.md <==
# WeefPy - Tiny Python web API framework

Tiny WSGI-Python framework designed mainly for REST API backends

## Installing WeefPy

Simply use pip for install this package
Installation:
```sh
$ pip install WeefPy
```
If you want to use development server install its dependencies
```sh
$ pip install uwsgi argparse python-daemon
```
## Running development server

Change directory to project folder and run devserver.

```sh
$ weef_server --host 127.0.0.1 --port 8080 --app `file`:`callable`
```
Replace `file` with filename with your main function and `callable` with your function. <br>
App should run on [localhost:8080](http://localhost:8080/).

#### Command parameters
- `--config` should contain uwsgi ini configuration file
- `--host` should contain IP address to be server binded on
- `--port` should contain port to be server binded on
- `--app` should contain file and callable object separated by colon. (main.py:main)
- `--pythonpath` should contain your code directory for imports in your project
- `--daemon` starts server in deamon mode

### Bash syntax:
```sh
$ weef_server [--config CONFIG] [--host IP_ADDRESS] [--port PORT] [--app file:callable] [--pythonpath path] [--daemon]
```

You can create server configuration in uwsgi ini file. <br>
If you want to run python files without devserver add project directory to `PYTHONPATH`.

## Using WeefPy

### Initializing

```py
from WeefPy import WSGI

application = WSGI()

```
#### WSGI parameters:
- auth_fn - specifies function called to perform request authorization (optional, default: empty function)
- middleware - provide function called before every route method (optional, default: None)
- content_type - this argument specifies the default content type of generated responses (optional, default: "application/json")
- status_codes - this argument specifies dictionary that is used to determine HTTP answers for 4xx & 5xx codes in different content types - by default is used text/html type which is mandatory if this argument is specified. For example watch [codes.json](https://gitlab.com/katry/WeefPy/-/blob/master/WeefPy/codes.json) (optional, default: [codes.json](https://gitlab.com/katry/WeefPy/-/blob/master/WeefPy/codes.json))

#### WSGI items:
- status_codes - dictionary of status codes (to see status_codes in WSGI parameters section)

##### Authorization function
The authorization function passed to the `WSGI object` is called with `Request` and `Response` parameters and returns boolean (True - authorized / False - unauthorized).

###### Authorization example
```py
from WeefPy import WSGI


def authorization(req, resp):
	if req.get_header("Authorization") == "my_simple_token":
		return True
	else:
		resp.set("BAD AUTH TOKEN", "401 UNAUTHORIZED") # you could set response here (optional)
		return False


application = WSGI(auth_fn=authorization, content_type="text/html")

```

### Routes

#### Route parameters
- endpoint - object should contain str or regex Pattern to match request path (mandatory)
- methods - list of HTTP methods to match request method - higher priority than method (optional, default: [])
- method - single HTTP method to match request method (optional, default: `GET`)
- auth_required - determines whether authorization will be performed before performing the route function (optional, defaul: False)

#### Using
Route is used as a decorator callable object you want to use as router method.
Router method takes two mandatory arguments `Request` and `Response` (explained below in a separate section).
Router method can also take variables from router regex match (if used).

#### Examples

```py
from WeefPy import Route


@Route("/endpoint", method="GET")  # simple route
def routed_method_1(request, response):  # simple route method
	return "Response from route 1" # response return without status code & mime type


# re package import needed
@Route(re.compile(r"/endpoint/(\d+)$"), methods=["GET", "POST"]) # regex route, with multiple methods
def routed_method_2(request, response):
	return "Response from route 2", "200 OK", "text/html" # full response return


@Route("/endpoint3", methods=["GET", "POST"]) #simple route, with multiple methods
def routed_method_2(request, response):
	response.set("Response by set") # alternatively can be called reponse method set to set response
```

### Request & Response objects

Instances of these object are given to every function that is called with `@Route` decorator
```py
@Route("/endpoint", method="GET")
def endpoint(Request, Response):  # instances of Request and Response objects
	pass
```

#### Request items and methods
- host - host recieved message (ip, domain etc.)
- method - request method (GET, POST etc.)
- path - request path
- query_string - request query_string
- data - request parsed data (json, -www-form-urlencoded, multipart/form-data)
- content_type - request content type
- content_length - request content length
- cookie - `SimpleCookie` object instance with request cookies loaded
<br><br>
- get_header(name) - get HTTP header by name

#### Response  items and methods
- res - `wsgi response` object
- encoding - response encoding; default: `UTF-8`
- headers - response headers
<br><br>
- get_body() - get response body
- get_status() - get response status
- get_content_type() - response content type - by default inherited from `WSGI object`
- set(body, status, content_type) - sets response (`body` (mandatory), `status` - by defaul is set to `200 OK`, `content_type` - if not provided is inherited from `WSGI object content_type`)
- redirect(location, status) - redirects response to (`location`, `status` if not providet is set to `302`)
- set_cookie(name, value, expires=None, maxAge=None, **kwargs) - adds cookie to response with obvious parameres, you can alwo add additional arguments (`kwargs`) such as `Domain`, `Path`, `Secure`, `HttpOnly`
- add_header(name, value) - adds header to response

## Example

Save this file as main.py
```py
from WeefPy import WSGI, Route

application = WSGI(content_type="text/html")


@Route("/endpoint", method="GET")
def endpoint(Request, Response):
	Response.add_header("X-header", "blahblah")  # set response header
	Response.set_cookie("cookie_from_server", "384")  # set reponse cookie
	return "Response BODY"
```

Now you should run weef_server
```sh
$ weef_server --host 127.0.0.1 --port 8080 --app main.py:application
```
App should run on [localhost:8080](http://localhost:8080/).<br>
http://localhost:8080/endpoint should now returns "Response BODY"
